@@ -152,6 +152,24 @@ sj_stock_init (void)
   initialized = TRUE;
 }
 
+static void
+set_busy_cursor (gboolean busy)
+{
+  if (!GTK_WIDGET_REALIZED (main_window))
+    return;
+
+  if (busy) {
+    GdkCursor *cursor;
+    cursor = gdk_cursor_new_for_display (gdk_drawable_get_display (main_window->window),
+					 GDK_WATCH);
+    gdk_window_set_cursor (main_window->window, cursor);
+    gdk_cursor_unref (cursor);
+    gdk_display_sync (gdk_drawable_get_display (main_window->window));
+  } else {
+    gdk_window_set_cursor (main_window->window, NULL);
+  }
+}
+
 void
 sj_main_set_title (const char* detail)
 {
@@ -208,17 +226,25 @@ void on_eject_activate (GtkMenuItem *item, gpointer user_data)
   nautilus_burn_drive_eject (drive);
 }
 
-gboolean poll_tray_opened (gpointer data)
+static gboolean poll_helper (gpointer data)
+{
+  reread_cd (TRUE);
+  return FALSE;
+}
+
+static gboolean poll_tray_opened (gpointer data)
 {
   gboolean new_status;
-
   if (extracting == TRUE)
     return TRUE;
 
   new_status = nautilus_burn_drive_door_is_open (drive);
   if (new_status != tray_opened && new_status == FALSE) {
-    reread_cd (TRUE);
+    d(g_printerr("Poll detected new CD\n"));
+    set_busy_cursor (TRUE);
+    g_timeout_add (300, poll_helper, NULL);
   } else if (new_status != tray_opened && new_status == TRUE) {
+    d(g_printerr("Poll detected removed CD\n"));
     stop_ui_hack ();
     update_ui_for_album (NULL);
   }
@@ -577,11 +603,10 @@ static void
 metadata_cb (SjMetadata *m, GList *albums, GError *error)
 {
   gboolean realized = GTK_WIDGET_REALIZED (main_window);
-
-  if (realized)
-    gdk_window_set_cursor (main_window->window, NULL);
-    /* Clear the statusbar message */
-    gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 0);
+  
+  set_busy_cursor (FALSE);
+  /* Clear the statusbar message */
+  gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 0);
   
   if (error && !(error->code == SJ_ERROR_CD_NO_MEDIA)) {
     GtkWidget *dialog;
@@ -650,27 +675,22 @@ is_audio_cd (NautilusBurnDrive *drive)
 static void reread_cd (gboolean ignore_no_media)
 {
   /* TODO: remove ignore_no_media? */
-  GError *error = NULL;
-  GdkCursor *cursor;
   gboolean realized = GTK_WIDGET_REALIZED (main_window);
+  GError *error = NULL;
   
   /* Set watch cursor */
-  if (realized) {
-    cursor = gdk_cursor_new_for_display (gdk_drawable_get_display (main_window->window),
-					 GDK_WATCH);
-    gdk_window_set_cursor (main_window->window, cursor);
-    gdk_cursor_unref (cursor);
-    gdk_display_sync (gdk_drawable_get_display (main_window->window));
-  }
+  set_busy_cursor (TRUE);
   
   /* Set statusbar message */
   gtk_statusbar_push(GTK_STATUSBAR(status_bar), 0, _("Retrieving track listing...please wait."));
 
+  d(if (!drive) g_printerr("Attempting to re-read NULL drive\n"));
+
   if (!is_audio_cd (drive)) {
+    d(g_printerr("Media is not an audio CD\n"));
     update_ui_for_album (NULL);
     gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 0);
-    if (realized)
-      gdk_window_set_cursor (main_window->window, NULL);
+    set_busy_cursor (FALSE);
     return;
   }
   
@@ -700,6 +720,43 @@ static void reread_cd (gboolean ignore_no_media)
   }
 }
 
+static void
+set_device(const char* device, gboolean ignore_no_media)
+{
+  if (device == NULL) {
+    drive = NULL;
+  } else if (access (device, R_OK) != 0) {
+    GtkWidget *dialog;
+    char *message;
+    message = g_strdup_printf (_("Sound Juicer could not access the CD-ROM device '%s'"), device);
+    dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_MESSAGE_ERROR,
+                                     GTK_BUTTONS_CLOSE,
+                                     "<b>%s</b>\n\n%s\n%s: %s",
+                                     _("Could not read the CD"),
+                                     message,
+                                     _("Reason"),
+                                     strerror (errno));
+    g_free (message);
+    gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
+    /* Set a null device */
+    drive = NULL;
+  } else {
+    drive = nautilus_burn_drive_new_from_path (device);
+    g_assert (drive);
+  }
+  sj_metadata_set_cdrom (metadata, device);
+  sj_extractor_set_device (extractor, device);
+  
+  tray_opened = nautilus_burn_drive_door_is_open (drive);
+  if (tray_opened == FALSE) {
+    reread_cd (ignore_no_media);
+  }
+}
+
 /**
  * The GConf key for the device changed
  */
@@ -731,36 +788,8 @@ void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, g
     drive = nautilus_burn_drive_new_from_path (device);
   } else {
     device = gconf_value_get_string (entry->value);
-    if (access (device, R_OK) != 0) {
-      GtkWidget *dialog;
-      char *message;
-      message = g_strdup_printf (_("Sound Juicer could not access the CD-ROM device '%s'"), device);
-      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
-                                       GTK_DIALOG_DESTROY_WITH_PARENT,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
-                                       "<b>%s</b>\n\n%s\n%s: %s",
-				       _("Could not read the CD"),
-				       message,
-				       _("Reason"),
-				       strerror (errno));
-      g_free (message);
-      gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-      /* Set a null device */
-      drive = NULL;
-    } else {
-      drive = nautilus_burn_drive_new_from_path (device);
-    }
-  }
-  sj_metadata_set_cdrom (metadata, device);
-  sj_extractor_set_device (extractor, device);
-
-  tray_opened = nautilus_burn_drive_door_is_open (drive);
-  if (tray_opened == FALSE) {
-    reread_cd (ignore_no_media);
   }
+  set_device (device, ignore_no_media);
 }
 
 void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
@@ -1277,8 +1306,11 @@ int main (int argc, char **argv)
   paranoia_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PARANOIA, NULL, TRUE, NULL), NULL);
   strip_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_STRIP, NULL, TRUE, NULL), NULL);
   eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
-  if (device == NULL)
+  if (device == NULL) {
     device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), GINT_TO_POINTER (TRUE));
+  } else {
+    set_device (device, TRUE);
+  }
 
   if (sj_extractor_supports_encoding (&error) == FALSE) {
     error_on_start (error);
